# coding=utf-8
import umfpayservice
from umfpayservice import error, util

import base64
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5 as Cipher_pkcs1_v1_5
from Crypto.Signature import PKCS1_v1_5 as Signature_pkcs1_v1_5
from Crypto.Hash import SHA

def encrypt_data(data):
    '''
    使用联动公钥加密数据
    '''
    key = get_public_key()
    rsakey = RSA.importKey(key)
    cipher = Cipher_pkcs1_v1_5.new(rsakey)
    text = cipher.encrypt(data)
    return base64.b64encode(text)

def sign(data, mer_id):
    '''
    使用商户私钥对数据签名
    '''
    gbk_data = util.deutf8(data).encode('gbk')

    private_key = get_private_key(mer_id)
    key = RSA.importKey(private_key)
    h = SHA.new(gbk_data)
    signer = Signature_pkcs1_v1_5.new(key)
    signature = signer.sign(h)
    return base64.b64encode(signature)

def app_sign(data, mer_id):
    '''
    app的签名方法
    '''
    base64_sign = sign(data, mer_id)
    return base64.b64decode(base64_sign).encode('hex')

def verify(data, signature):
    '''
    使用联动公钥对数据验签
    '''
    public_key = get_public_key()
    key = RSA.importKey(public_key)
    digest = SHA.new()
    digest.update(util.deutf8(data).encode('gbk'))
    verifier = Signature_pkcs1_v1_5.new(key)

    sig = base64.b64decode(signature)
    if verifier.verify(digest, sig):
        return True
    return False


def get_private_key(mer_id):
    '''
    获取商户私钥
    '''
    private_key_path = umfpayservice.umf_config.mer_private_keys[mer_id]
    if private_key_path is None:
        raise error.ConfigError("[配置错误] mer_id不在商户私钥配置列表中。(mer_id: %s)"
                                "商户私钥配置列表为：%s. " % (mer_id, umfpayservice.umf_config.mer_private_keys), "mer_id", mer_id)

    private_key = open(private_key_path, "r").read()
    return private_key.strip()

def get_public_key():
    '''获取联动公钥'''
    return umfpayservice.umf_config.umf_public_key

def get_sorted_plain(params):
    params_copy = params.copy()
    # 删除不参与签名的字段
    if 'sign_type' in params_copy:
        del params_copy['sign_type']
    if 'sign' in params_copy:
        del params_copy['sign']

    params_copy = sorted(list(params_copy.items()), key=lambda d: d[0])
    plain = util.convert_url_not_encode(params_copy)

    # util.logger.info("获取签名明文串：%s" % plain)
    return plain


# -*- coding: utf-8 -*-

import sys
import textwrap

from umfpayservice import util, error

# - Requests是优先级最高的HTTP请求库
# - 谷歌的应用程序引擎使用urlfetch
# - Pycurl是第三优先级的库，毕竟它可以进行SSL认证。
# - 回落到urlurllib2.
try:
    import urllib2
except ImportError:
    pass

try:
    import pycurl
except ImportError:
    pycurl = None

try:
    import requests
except ImportError:
    requests = None
else:
    try:
        # Require version 0.8.8, but don't want to depend on distutils
        version = requests.__version__
        major, minor, patch = [int(i) for i in version.split('.')]
    except Exception:
        # Probably some new-fangled version, so it should support verify
        pass
    else:
        if (major, minor, patch) < (0, 8, 8):
            util.logger.warning('警告：你的requests库版本为：%s。'
                                'umfservice库需要你的requests库的版本大于0.8.8。'
                                'umfservice会使用另外的HTTP请求库以便使umfservice正常工作。\n'
                                '我们建议你升级你的requests库')
            requests = None

try:
    from google.appengine.api import urlfetch
except ImportError:
    urlfetch = None


def new_default_http_client(*args, **kwargs):
    if urlfetch:
        impl = UrlFetchClient
    elif requests:
        impl = RequestsClient
    elif pycurl:
        impl = PycurlClient
    else:
        impl = Urllib2Client

        util.logger.warning('由于没有requests和pycurl库，umfservice请求库已回落到urllib2/urllib'
                            'urllib2没有SSL验证服务器证书，出于安全考虑，请安装requests库')

    return impl(*args, **kwargs)


class HTTPClient(object):

    def __init__(self, verify_ssl_certs=True):
        self._verify_ssl_certs = verify_ssl_certs

    def request(self, method, url, headers, post_data=None):
        raise NotImplementedError(
            'HTTPClient子类必须重写request方法')


class RequestsClient(HTTPClient):
    name = 'requests'

    def request(self, method, url, headers, post_data=None):
        kwargs = {}

        try:
            try:
                result = requests.request(method,
                                          url,
                                          headers=headers,
                                          data=post_data,
                                          timeout=80,
                                          **kwargs)
            except TypeError as e:
                raise error.RequestError(
                    'requests库版本与umfservice不兼容，请更新requests库。')

            content = result.content
            status_code = result.status_code
        except Exception as e:
            self._handle_request_error(e)
        return content, status_code

    def _handle_request_error(self, e):
        if isinstance(e, requests.exceptions.RequestException):
            msg = ("网络异常：：：  ")
            err = "%s: %s" % (type(e).__name__, str(e))
        else:
            msg = ("网络异常：：： "
                   "可能是配置错误导致，")
            err = "%s异常被抛出，" % (type(e).__name__,)
            if str(e):
                err += " 错误信息为： %s。" % (str(e),)
            else:
                err += " 没有错误信息。"
        msg = textwrap.fill(msg) + "\n\n(网络错误: %s)" % (err,)
        raise error.RequestError(msg)


class UrlFetchClient(HTTPClient):
    name = 'urlfetch'

    def request(self, method, url, headers, post_data=None):
        try:
            result = urlfetch.fetch(
                url=url,
                method=method,
                headers=headers,
                validate_certificate=self._verify_ssl_certs,
                deadline=55,
                payload=post_data
            )
        except urlfetch.Error as e:
            self._handle_request_error(e, url)

        return result.content, result.status_code

    def _handle_request_error(self, e, url):
        msg = '网络异常：：：'
        msg = textwrap.fill(msg) + "\n\n(Network error: " + str(e) + ")"
        raise error.RequestError(msg)


class PycurlClient(HTTPClient):
    name = 'pycurl'

    def request(self, method, url, headers, post_data=None):
        s = util.StringIO.StringIO()
        curl = pycurl.Curl()

        if method == 'get':
            curl.setopt(pycurl.HTTPGET, 1)
        elif method == 'post':
            curl.setopt(pycurl.POST, 1)
            curl.setopt(pycurl.POSTFIELDS, post_data)
        else:
            curl.setopt(pycurl.CUSTOMREQUEST, method.upper())

        # pycurl doesn't like unicode URLs
        curl.setopt(pycurl.URL, util.utf8(url))

        curl.setopt(pycurl.WRITEFUNCTION, s.write)
        curl.setopt(pycurl.NOSIGNAL, 1)
        curl.setopt(pycurl.CONNECTTIMEOUT, 30)
        curl.setopt(pycurl.TIMEOUT, 80)
        curl.setopt(pycurl.HTTPHEADER, ['%s: %s' % (k, v)
                    for k, v in headers.items()])

        curl.setopt(pycurl.SSL_VERIFYHOST, False)

        try:
            curl.perform()
        except pycurl.error as e:
            self._handle_request_error(e)
        rbody = s.getvalue()
        rcode = curl.getinfo(pycurl.RESPONSE_CODE)
        return rbody, rcode

    def _handle_request_error(self, e):
        msg = "网络异常：：："
        msg = textwrap.fill(msg) + "\n\n(Network error: " + e[1] + ")"
        raise error.RequestError(msg)

class Urllib2Client(HTTPClient):
    if sys.version_info >= (3, 0):
        name = 'urllib.request'
    else:
        name = 'urllib2'

    def request(self, method, url, headers, post_data=None):
        if sys.version_info >= (3, 0) and isinstance(post_data, str):
            post_data = post_data.encode('utf-8')

        req = urllib2.Request(url, post_data, headers)

        if method not in ('get', 'post'):
            req.get_method = lambda: method.upper()

        try:
            response = urllib2.urlopen(req)
            rbody = response.read()
            rcode = response.code
        except urllib2.HTTPError as e:
            rcode = e.code
            rbody = e.read()
        except (urllib2.URLError, ValueError) as e:
            self._handle_request_error(e)
        return rbody, rcode

    def _handle_request_error(self, e):
        msg = ("网络异常：：：")
        msg = textwrap.fill(msg) + "\n\n(Network error: " + str(e) + ")"
        raise error.RequestError(msg)



# -*- coding: utf-8 -*-

import os

import umfpayservice
from umfpayservice import api_requestor, util, error, regex_check, sign, common

class UmfPayObject(dict):
    def __init__(self, service=None):
        super(UmfPayObject, self).__init__()

        self.service = service

class APIResource(UmfPayObject):

    def __init__(self, service=None):
        super(APIResource, self).__init__(service)

        if service in common.service_keys:
            self.service = service
        else:
            raise error.ParamsError("[请求参数校验] "
                                            "service: 为空或该接口本SDK不支持，请联系联动运营人员！")
        self.request_params = {}
        self.resp = {}

    def request(self, params):
        self.request_params = params

        requestor = api_requestor.APIRequestor()
        response = requestor.request('post', params)
        self.resp = response
        return response

    def create(self, **params):
        '''执行接口入口'''
        util.logger.info("\n\n开始执行===[%s]接口,service=%s ==========================="
                         % (common.logger_decs[self.service], self.service))

        prepared_params = self.check_do_params(params)
        if prepared_params is not None:
            response = self.request(prepared_params)

            util.logger.info("执行结束===[%s]接口,service=%s ==========================="
                             % (common.logger_decs[self.service], self.service))
            return response
        return None

    def check_do_params(self, params):
        '''校验和处理参数'''
        # util.logger.info("传入的参数列表为：\n%s" % util.JSON.str(params))

        params = self.add_common_params(params)
        params = {key: value.strip() for key, value in params.items()}

        if self.check_params(params):
            self.process_params(params)
            return params
        return None

    def add_common_params(self, params):
        '''
        添加公共参数
        '''
        common_params = {
            'service': self.service,
            'sign_type': umfpayservice.umf_config.sign_type,
            'charset': umfpayservice.umf_config.charset,
            'res_format': umfpayservice.umf_config.res_format,
            'version': umfpayservice.umf_config.api_version
        }
        # util.logger.info("添加公共参数列表为：\n%s" % util.JSON.str(common_params))
        return dict(common_params, **params)

    def check_params(self, params):
        '''校验参数'''

        # 必传字段校验
        required_keys = self.get_required_keys()
        util.logger.info("该接口的必传参数列表为：%r" % required_keys)
        for required_key in required_keys:
            if required_key not in params:
                raise error.ParamsError("[请求参数校验]"
                                        "该接口缺少必传字段。必传字段列表为：%s" % required_keys)

        # 正则校验
        if (regex_check.check(key, value) for key, value in params.items()):
            util.logger.info("参数校验通过！")
            return True
        return False

    @classmethod
    def process_params(cls, params):
        # 空参数过滤
        for key in list(params.keys()):
            if params[key] is None or (params[key] == ''):
                del params[key]

        # 敏感字段加密
        cls.encrypt_fields(params)

        # 添加签名
        try:
            plain = sign.get_sorted_plain(params)
            sign_str = sign.sign(plain, params['mer_id'])
            util.logger.info("生成签名为 sign：\n%s" % sign_str)
            params['sign'] = sign_str

            util.logger.info("处理后的请求参数为：\n%s" % str(params))
        except:
            raise error.SignatureError("使用商户私钥生成签名失败。")

    @classmethod
    def encrypt_fields(cls, params):
        for key, value in params.items():
            if (value is not None or '') and key in common.encrypt_fields:
                try:
                    value = value.decode('utf-8').encode('gbk')
                    params[key] = sign.encrypt_data(value)
                    util.logger.info("需要加密敏感字段：%s"
                                     "\n加密后为%s: %s" % (key, key, params[key]))
                except:
                    raise error.SignatureError("%s: 敏感字段加密失败。(%s: %s)" % (key, key, value))

    @classmethod
    def get_required_keys(cls):
        raise TypeError('APIResource 是一个虚拟类，你需要在它的子类中实现get_required_keys方法')


class DownLoadAPIResource(APIResource):

    def __init__(self, service=None):
        super(DownLoadAPIResource, self).__init__(service)

    def download(self, **params):

        prepared_params = self.check_do_params(params)
        if prepared_params is not None:
            file_path, file_name = self.get_save_path(prepared_params)

            util.logger.info("传入的保存文件路径为：\n%s\n文件名为：\n%s" % (file_path, file_name))
            rbody, rcode = api_requestor.APIRequestor().download(file_path, file_name, 'post', prepared_params)

            if not (200 <= rcode < 300):
                raise error.APIError(
                    "接口返回的响应错误: %r (响应码为： %d)" % (rbody, rcode),
                    rbody, rcode)
            else:
                self.write_file(file_path, file_name, self.do_response(rbody))
                return "下载对账文件成功，保存地址为：%s/%s" % (file_path, file_name)

    @classmethod
    def write_file(self, file_path, file_name, content):
        if not os.path.isdir(file_path):
            os.mkdir(file_path)

        try:
            content = content.decode('gbk').encode('utf-8')
            with open("%s/%s" % (file_path, file_name), 'w') as f:
                f.write(content)
            util.logger.info("文件下载成功，文件路径为：%s/%s" % (file_path, file_name))
        except IOError as e:
            raise IOError("本地写文件失败，path= %s/%s" % (file_path, file_name))

    def get_save_path(self, params):
        raise TypeError('DownLoadAPIResource 是一个虚拟类，你需要在它的子类中实现get_save_path方法')

    def do_response(self, rbody):
        raise TypeError('DownLoadAPIResource 是一个虚拟类，你需要在它的子类中实现do_response方法')

class ParamsGetAPIResource(APIResource):

    def __init__(self, service=None):
        super(ParamsGetAPIResource, self).__init__(service)

    def create(self, **params):
        prepared_params = self.check_do_params(params)

        if prepared_params is not None:
            return self.params_to_get(prepared_params)
        return None

    def params_to_get(self, params):
        raise TypeError('ParamsGetAPIResource 是一个虚拟类，你需要在它的子类中实现params_to_get方法')







# -*- coding: utf-8 -*-
# umfpayservice Python
# Configurations

from umfpayservice.common import (
    encrypt_fields,
    Config
)
from umfpayservice.api_requestor import APIRequestor  # noqa

from umfpayservice.error import (  # 错误
    UmfPayError,
    ParamsError,
    SignatureError,
    APIError,
    RequestError)

from umfpayservice.util import logger  # 工具

from umfpayservice.umfservice import (  # 封装的请求接口类
    MobileOrder,
    ActiveScanPayment,
    PassiveScanPayment,
    QuickPayOrder,
    QuickGetMessage,
    QuickPayConfirm,
    QuickQuerybankSupport,
    QuickCancelSurrender,
    QueryhistoryOrder,
    QueryTodayOrder,
    CancelTrade,
    GeneralRefund,
    MassTransferRefund,
    QueryRefundState,
    RemedyRefundInformation,
    PaymentOrder,
    QueryPaymentStatus,
    QueryAccountBalance,
    DebitCardAuthentication,
    CreditCardAuthentication,
    IdentityAuthentication,
    ReconciliationDownload,
    AsynNotification,
    WebFrontPagePay,
    H5FrontPage,
    PublicPayment,
    UmfServiceUtils
    )

from umfpayservice.resource import (  # noqa
    UmfPayObject,
    APIResource,
    DownLoadAPIResource,
    ParamsGetAPIResource
)

umf_config = Config()




# -*- coding: utf-8 -*-

import urllib
from bs4 import BeautifulSoup

import umfpayservice
from umfpayservice import http_client, util, sign, error

# 请求联动服务地址
service_url = 'https://pay.soopay.net/spay/pay/payservice.do'

def api_encode(data):
    '''
    api请求参数编码处理
    :param data: 发送请求的数据
    :return:
    '''
    for key, value in data.items():
        yield (util.utf8(key), util.utf8(value))

class APIRequestor(object):

    def __init__(self, client=None):
        self._client = client or http_client.new_default_http_client(
            verify_ssl_certs=False)

    def request(self, method, params=None):
        rbody, rcode = self.request_raw(
            method.lower(), params)
        resp = self.interpret_response(rbody, rcode)
        return resp

    def download(self, path, name, method, params=None):
        rbody, rcode = self.request_raw(
            method.lower(), params)
        return rbody, rcode

    def request_raw(self, method, params=None):
        post_data = util.url_encode(list(api_encode(params or {})))

        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Content-Encoding': umfpayservice.umf_config.charset
        }
        util.logger.info("请求前数据：post_data= %s" % post_data)
        rbody, rcode = self._client.request(
            method, service_url, headers, post_data)

        util.logger.info('请求后响应：'
            '\n返回rcode: %d '
            '\n返回rbody: %s ', rcode, rbody)
        return rbody, rcode

    def interpret_response(self, rbody, rcode):
        '''
        解析response
        :param rbody:
        :param rcode:
        :return:
        '''
        if not (200 <= rcode < 300):
            raise error.APIError(
                "接口返回的响应错误: %r (响应码为： %d)" % (rbody, rcode),
                rbody, rcode)
        try:
            rbody_dict = self._interpret_meta(rbody.strip())
            if sign.verify(rbody_dict['plain'], rbody_dict['sign']):
                if 'sign' in rbody_dict:
                    del rbody_dict['sign']
                if 'sign_type' in rbody_dict:
                    del rbody_dict['sign_type']
                if 'version' in rbody_dict:
                    del rbody_dict['version']
                if 'plain' in rbody_dict:
                    del rbody_dict['plain']
                resp = rbody_dict

                util.logger.info("解析响应数据获得：\n%s" % str(resp))
                return resp
        except Exception as e:
            raise error.APIError(
                "接口返回的响应错误: %r (响应码为： %d)" % (rbody, rcode),
                rbody, rcode)
        return None

    def _interpret_meta(self, rbody):
        '''
        解析meta
        :param rbody:
        :return:
        '''
        try:
            soup = BeautifulSoup(rbody, "html.parser")
            content = soup.head.meta['content']
            content = util.utf8(content)

            resp = {param_key: param_value for param_key, param_value in (split_param.split('=', 1) for split_param in content.split('&'))}
            plain = sign.get_sorted_plain(resp)
            resp['plain'] = plain
        except:
            raise error.RequestError('解析服务器响应失败。body = %s' % rbody)
        return resp



# coding=utf-8
import logging

from umfpayservice import error, regex_check
from umfpayservice.util import logger
import datetime
import os

class Config(object):

    def __init__(self):
        # # log文件输出路径
        # self.log_path = "./logs"
        # 私钥对
        self.mer_private_keys = {}
        # 联动公钥
        self.umf_public_key = '-----BEGIN PUBLIC KEY-----\n' \
                              'MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDGWAu4p6aK1SiQqNKT1nTgYwA8\n' \
                              'cz0Rde9fRtmLJAx1QxLqrerAUVl/uuXV7NQFSkTipouo3cwEEpae89267AeLJBzK\n' \
                              'PbKnUID6JYGbwnq7CiRR4E244zcgqE8jo8DnkbH3KkiWonoUMD1uHy6TUFv5W7zr\n' \
                              'haz/E59MVmbzrp1TwwIDAQAB\n' \
                              '-----END PUBLIC KEY-----'
        # 接口版本
        self.api_version = '4.0'
        self.auth_version = '1.0'
        # 默认签名加密方式
        self.sign_type = 'RSA'
        # 默认编码方式
        self.charset = 'UTF-8'
        # 默认响应格式
        self.res_format = 'HTML'

        self.is_log = True
        self.log_path = './logs'
        self.log_level = logging.INFO

    # def __setattr__(self, key, value):
    #     if not hasattr(self, key):
    #         logger.warning("umfservice的umf_config配置不允许添加新的配置项。添加或修改配置失败。(%s: %s)", key, value)
    #     else:
    #         object.__setattr__(self, key, value)

    def add_private_keys(self, key_list=None):
        '''
        动态添加商户号和私钥路径对
        :param key_list:
        :return:
        '''
        if key_list is None:
            return

        if isinstance(key_list, list):
            for mer_id, private_key_path in key_list:
                if regex_check.check("mer_id", mer_id):
                    self.mer_private_keys[mer_id] = private_key_path
                else:
                    raise error.ConfigError("[配置错误] mer_id格式不匹配。(mer_id: %s)"
                                            "匹配规则为：%s" % (mer_id, regex_check.get_regexs()["mer_id"]))

    def set_log_path(self, log_path, level=logging.INFO):
        '''设置log路径'''
        self.is_log = True
        self.log_path = log_path
        self.log_level = level
        self.set_logger()

    def set_log(self, is_log):
        self.is_log = is_log
        self.set_logger()

    def set_logger(self):
        if self.is_log and logger.disabled is 0:
            if os.path.isdir(self.log_path):
                pass
            else:
                os.mkdir(self.log_path)
            log_file = "%s/umfservice_log_%s.log" % (self.log_path, datetime.datetime.now().strftime("%Y%m%d"))

            fh = logging.FileHandler(log_file)
            fh.setLevel(logging.DEBUG)

            formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - [UMF SDK]%(message)s')
            fh.setFormatter(formatter)
            logger.setLevel(self.log_level)
            logger.addHandler(fh)
        elif not self.is_log:
            logger.disabled = True


# 需要加密的敏感字段
encrypt_fields = ["card_id", "valid_date", "cvv2", "pass_wd", "identity_code",
                  "card_holder", "recv_account", "recv_user_name", "identity_holder",
                  "identityCode", "cardHolder", "mer_cust_name", "account_name", "bank_account", "endDate"]

# 联动提供的service列表
service_keys = ["pay_req", "pay_req_ivr_call", "pay_req_ivr_tcall", "query_order", "mer_cancel",
                    "mer_refund", "download_settle_file", "pay_req_split_front", "pay_req_split_back",
                    "pay_req_split_direct", "split_refund_req", "pay_result_notify", "split_req_result",
                    "split_refund_result", "credit_direct_pay", "debit_direct_pay", "pre_auth_direct_req",
                    "pre_auth_direct_pay", "pre_auth_direct_cancel", "pre_auth_direct_query",
                    "pre_auth_direct_refund", "pre_auth_direct_settle", "pay_transfer_register",
                    "pay_transfer_req", "pay_transfer_order_query", "pay_transfer_mer_refund", "card_auth",
                    "req_sms_verifycode", "pay_confirm", "pay_req_shortcut_front", "pay_req_shortcut",
                    "agreement_pay_confirm_shortcut", "query_mer_bank_shortcut", "unbind_mercust_protocol_shortcut",
                    "split_req",  "query_split_order",  "transfer_direct_req",  "transfer_query",
                    "mer_order_info_query",  "mer_refund_query",  "active_scancode_order", "passive_scancode_pay",
                    "query_account_balance", "comm_auth", "apply_pay_shortcut", "sms_req_shortcut",
                    "confirm_pay_shortcut", "get_mer_bank_shortcut", "unbind_mercust_protocol_shortcut",
                    "refund_info_replenish", "req_bind_verify_shortcut","req_bind_confirm_shortcut",
                    "bind_agreement_notify_shortcut","bind_req_shortcut_front", "req_front_page_pay"]

# service对应的日志描述，log用
logger_decs = {
    # service: (接口名字/..)
    "pay_req": ("APP支付下单"),
    "pay_req_ivr_call": (""),
    "pay_req_ivr_tcall": (""),
    "query_order": ("订单查询---查询当天订单"),
    "mer_cancel": ("撤销"),
    "mer_refund": ("退款---普通退款"),
    "download_settle_file": ("对账---对账文件下载"),
    "pay_req_split_front": (""),
    "pay_req_split_back": (""),
    "pay_req_split_direct": (""),
    "split_refund_req": ("退款---批量转账退费"),
    "pay_result_notify": (""),
    "split_req_result": (""),
    "split_refund_result": (""),
    "credit_direct_pay": (""),
    "debit_direct_pay": (""),
    "pre_auth_direct_req": (""),
    "pre_auth_direct_pay": (""),
    "pre_auth_direct_cancel": (""),
    "pre_auth_direct_query": (""),
    "pre_auth_direct_refund": (""),
    "pre_auth_direct_settle": (""),
    "pay_transfer_register": (""),
    "pay_transfer_req": (""),
    "pay_transfer_order_query": (""),
    "pay_transfer_mer_refund": (""),
    "card_auth": (""),
    "req_sms_verifycode": (""),
    "pay_confirm": (""),
    "pay_req_shortcut_front": (""),
    "pay_req_shortcut": (""),
    "agreement_pay_confirm_shortcut": (""),
    "query_mer_bank_shortcut": ("收款---快捷支付获取银行卡列表"),
    "unbind_mercust_protocol_shortcut": ("收款---快捷支付解约."),
    "split_req": (""),
    "query_split_order": (""),
    "transfer_direct_req": ("付款---下单"),
    "transfer_query": ("付款---付款状态查询"),
    "mer_order_info_query": ("订单查询---查询历史订单"),
    "mer_refund_query": ("退款---退款状态查询方法"),
    "active_scancode_order": ("收款---扫码支付（主扫）下单方法"),
    "passive_scancode_pay": ("收款---扫码支付（被扫）下单."),
    "query_account_balance": ("付款---余额查询"),
    "comm_auth": ("鉴权"),
    "apply_pay_shortcut": ("收款---快捷支付下单."),
    "sms_req_shortcut": ("收款---快捷支付向平台获取短信验证码."),
    "confirm_pay_shortcut": ("收款---快捷支付确认支付"),
    "get_mer_bank_shortcut": (""),
    "refund_info_replenish": ("退款---退费信息补录"),
    "req_bind_verify_shortcut": (""),
    "req_bind_confirm_shortcut": (""),
    "bind_agreement_notify_shortcut": (""),
    "bind_req_shortcut_front": (""),
    "req_front_page_pay": ("Web收银台---生成get后的请求参数"),
    "pay_req_h5_frontpage": ("H5收银台---生成get后的请求参数"),
    "publicnumber_and_verticalcode": ("公众号支付--生成get后的请求参数"),
}




# coding=utf-8

import datetime
import urllib

import umfpayservice
from umfpayservice import util, error, sign
from umfpayservice.resource import APIResource, UmfPayObject, DownLoadAPIResource, ParamsGetAPIResource

class MobileOrder(APIResource):
    '''APP支付下单'''
    def __init__(self):
        super(MobileOrder, self).__init__('pay_req')

    @classmethod
    def get_required_keys(cls):
        return ["service" ,"charset", "mer_id", "sign_type", "version", "order_id", "mer_date", "amount", "amt_type"]

class ActiveScanPayment(APIResource):
    '''收款---扫码支付（主扫）下单方法'''
    def __init__(self):
        super(ActiveScanPayment, self).__init__('active_scancode_order')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "notify_url", "version", "goods_inf", "order_id", "mer_date", "amount", "amt_type", "scancode_type"]

class PassiveScanPayment(APIResource):
    '''收款---扫码支付（被扫）下单.'''
    def __init__(self):
        super(PassiveScanPayment, self).__init__('passive_scancode_pay')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "notify_url", "version", "goods_inf", "order_id", "mer_date", "amount", "amt_type", "auth_code", "use_desc", "scancode_type"]

class QuickPayOrder(APIResource):
    '''收款---快捷支付下单.'''
    def __init__(self):
        super(QuickPayOrder, self).__init__('apply_pay_shortcut')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "version", "order_id", "notify_url", "order_id", "mer_date", "amount", "amt_type", "pay_type", "gate_id"]

class QuickGetMessage(APIResource):
    '''收款---快捷支付向平台获取短信验证码.'''
    def __init__(self):
        super(QuickGetMessage, self).__init__('sms_req_shortcut')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "version", "trade_no", "media_id", "media_type"]

class QuickPayConfirm(APIResource):
    '''收款---快捷支付确认支付'''
    def __init__(self):
        super(QuickPayConfirm, self).__init__('confirm_pay_shortcut')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "version", "trade_no", "trade_no", "verify_code", "media_type", "media_id"]

class QuickQuerybankSupport(APIResource):
    '''收款---快捷支付获取银行卡列表'''
    def __init__(self):
        super(QuickQuerybankSupport, self).__init__('query_mer_bank_shortcut')

    @classmethod
    def get_required_keys(cls):
        return ["service", "sign_type", "charset", "mer_id", "version", "pay_type"]

class QuickCancelSurrender(APIResource):
    '''收款---快捷支付解约.'''
    def __init__(self):
        super(QuickCancelSurrender, self).__init__('unbind_mercust_protocol_shortcut')

    @classmethod
    def get_required_keys(cls):
        return ["service", "sign_type", "charset", "mer_id", "version"]

class QueryhistoryOrder(APIResource):
    '''订单查询---查询历史订单'''
    def __init__(self):
        super(QueryhistoryOrder, self).__init__('mer_order_info_query')

    @classmethod
    def get_required_keys(cls):
        return ["service", "sign_type", "charset", "mer_id", "version", "order_id", "mer_date"]

class QueryTodayOrder(APIResource):
    '''订单查询---查询当天订单'''
    def __init__(self):
        super(QueryTodayOrder, self).__init__('query_order')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "sign_type", "mer_id", "version", "order_id", "mer_date"]

class CancelTrade(APIResource):
    '''撤销'''
    def __init__(self):
        super(CancelTrade, self).__init__('mer_cancel')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "sign_type", "mer_id", "version", "order_id", "mer_date", "amount"]


class GeneralRefund(APIResource):
    '''退款---普通退款'''
    def __init__(self):
        super(GeneralRefund, self).__init__('mer_refund')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "sign_type", "mer_id", "version", "refund_no", "order_id", "mer_date", "org_amount"]

class MassTransferRefund(APIResource):
    '''退款---批量转账退费'''
    def __init__(self):
        super(MassTransferRefund, self).__init__('split_refund_req')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "version", "refund_no", "order_id", "mer_date", "refund_amount", "org_amount", "sub_mer_id", "sub_order_id"]

class QueryRefundState(APIResource):
    '''退款---退款状态查询方法'''
    def __init__(self):
        super(QueryRefundState, self).__init__('mer_refund_query')

    @classmethod
    def get_required_keys(cls):
        return ["service", "sign_type", "charset", "mer_id", "version", "refund_no"]

class RemedyRefundInformation(APIResource):
    '''退款---退费信息补录'''
    def __init__(self):
        super(RemedyRefundInformation, self).__init__('refund_info_replenish')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "version", "refund_no", "card_holder", "card_id"]

#------------------------------------------------------------
# 付款
#------------------------------------------------------------
class PaymentOrder(APIResource):
    '''付款---下单'''
    def __init__(self):
        super(PaymentOrder, self).__init__('transfer_direct_req')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "version", "sign_type", "order_id", "mer_date", "amount", "recv_account_type", "recv_bank_acc_pro", "recv_account", "recv_user_name", "recv_gate_id", "purpose", "prov_name", "city_name", "bank_brhname"]

class QueryPaymentStatus(APIResource):
    '''付款---付款状态查询'''
    def __init__(self):
        super(QueryPaymentStatus, self).__init__('transfer_query')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "version", "sign_type", "order_id", "mer_date"]

class QueryAccountBalance(APIResource):
    '''付款---余额查询'''
    def __init__(self):
        super(QueryAccountBalance, self).__init__('query_account_balance')

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "version", "sign_type"]

# ------------------------------------------------------------
# 鉴权
#------------------------------------------------------------
class DebitCardAuthentication(APIResource):
    '''鉴权---借记卡实名认证'''
    def __init__(self):
        super(DebitCardAuthentication, self).__init__('comm_auth')

    def add_common_params(self, params):
        common_params = {
            'service': self.service.strip(),
            'sign_type': umfpayservice.umf_config.sign_type.strip(),
            'charset': umfpayservice.umf_config.charset.strip(),
            'res_format': umfpayservice.umf_config.res_format.strip(),
            'version': umfpayservice.umf_config.auth_version.strip()
        }
        return dict(common_params, **params)

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "version", "auth_type", "order_id"]

class CreditCardAuthentication(APIResource):
    '''鉴权---贷记卡实名认证'''
    def __init__(self):
        super(CreditCardAuthentication, self).__init__('comm_auth')

    def add_common_params(self, params):
        common_params = {
            'service': self.service.strip(),
            'sign_type': umfpayservice.umf_config.sign_type.strip(),
            'charset': umfpayservice.umf_config.charset.strip(),
            'res_format': umfpayservice.umf_config.res_format.strip(),
            'version': umfpayservice.umf_config.auth_version.strip()
        }
        return dict(common_params, **params)

    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "version", "auth_type", "order_id"]

class IdentityAuthentication(APIResource):
    '''鉴权---身份认证'''
    def __init__(self):
        super(IdentityAuthentication, self).__init__('comm_auth')

    def add_common_params(self, params):
        common_params = {
            'service': self.service.strip(),
            'sign_type': umfpayservice.umf_config.sign_type.strip(),
            'charset': umfpayservice.umf_config.charset.strip(),
            'res_format': umfpayservice.umf_config.res_format.strip(),
            'version': umfpayservice.umf_config.auth_version.strip()
        }
        return dict(common_params, **params)


    @classmethod
    def get_required_keys(cls):
        return ["service", "charset", "mer_id", "sign_type", "version", "auth_type", "order_id"]

# ---------------------------------------------------------------
# 对账
# ---------------------------------------------------------------
class ReconciliationDownload(DownLoadAPIResource):
    '''对账---对账文件下载'''
    def __init__(self):
        super(ReconciliationDownload, self).__init__('download_settle_file')

    @classmethod
    def get_required_keys(cls):
        return ["service", "sign_type", "mer_id", "version", "settle_date"]

    def get_save_path(self, params):
        if 'settle_path' in params:
            file_path = params['settle_path']
            del params['settle_path']

            file_name = "settle_file_%s_%s.txt" % (params['mer_id'], datetime.datetime.now().strftime("%Y%m%d"))
            return file_path, file_name
        else:
            raise error.ParamsError("[请求参数校验] "
                                    "settle_path: 该字段不能为空。")

    def do_response(self, rbody):
        return rbody

# ------------------------------------------------------------
# 异步通知
# ------------------------------------------------------------

class AsynNotification(UmfPayObject):
    '''
    异步通知
    '''
    def __init__(self):
        super(AsynNotification, self).__init__()

    def notify_data_parser(self, notify_params_str):
        '''
        解析异步通知数据
        :param notify_params_str:
        :return:
        '''
        try:
            notify_params = {key: urllib.unquote(value) for key, value in
                            (split_param.split('=', 1) for split_param in notify_params_str.split('&'))}
            plain = sign.get_sorted_plain(notify_params)
            verify = sign.verify(plain, notify_params['sign'])
        except:
            raise error.SignatureError('异步通知验证签名异常')

        if verify:
            util.logger.info("平台通知数据验签成功")
            if 'sign' in notify_params: del notify_params['sign']
            if 'sign_type' in notify_params: del notify_params['sign_type']
            if 'version' in notify_params: del notify_params['version']

            return notify_params
        else:
            util.logger.info("平台通知数据验证签名发生异常")

    def response_umf_map(self, notify_params):
        '''
        拼接响应给平台的数据
        :param notify_params:
        :return:
        '''
        response_dict = {
            'mer_date': notify_params['mer_date'],
            'mer_id': notify_params['mer_id'],
            'order_id': notify_params['order_id'],
            'ret_code': '0000',
            'ret_msg': 'success',
            'version': '4.0',
            'sign_type': 'RSA'
        }

        mer_plain = sign.get_sorted_plain(response_dict)
        mer_sign = sign.sign(mer_plain, notify_params['mer_id'])
        response_dict['sign'] = mer_sign

        return util.convert_url_not_encode(response_dict)

class WebFrontPagePay(ParamsGetAPIResource):
    '''Web收银台---生成get后的请求参数，商户只需要拼接URL进行get请求即可'''
    def __init__(self):
        super(WebFrontPagePay, self).__init__('req_front_page_pay')

    def params_to_get(self, params):
        return urllib.urlencode(params)

    @classmethod
    def get_required_keys(cls):
        return ["mer_id", "order_id", "mer_date", "amount", "amt_type", "interface_ty"]

class H5FrontPage(ParamsGetAPIResource):
    '''H5收银台---生成get后的请求参数，商户只需要拼接URL进行get请求即可'''
    def __init__(self):
        super(H5FrontPage, self).__init__('pay_req_h5_frontpage')

    def params_to_get(self, params):
        return urllib.urlencode(params)

    @classmethod
    def get_required_keys(cls):
        return ["mer_id", "order_id", "mer_date", "notify_url", "ret_url", "amount", "amt_type"]

class PublicPayment(ParamsGetAPIResource):
    '''公众号支付--生成get后的请求参数，商户只需要拼接URL进行get请求即可'''
    def __init__(self):
        super(PublicPayment, self).__init__('publicnumber_and_verticalcode')

    def params_to_get(self, params):
        return urllib.urlencode(params)

    @classmethod
    def get_required_keys(cls):
        return ["mer_id", "order_id", "notify_url", "mer_date", "goods_inf", "amount", "amt_type", "is_public_number"]

class UmfServiceUtils(UmfPayObject):
    '''
    其他工具
    '''
    def __init__(self):
        super(UmfServiceUtils, self).__init__()

    @classmethod
    def generate_sign(cls, **params):
        '''SDK生成签名方法，传入请求字典，生成sign字段'''
        plain = sign.get_sorted_plain(params)
        return sign.sign(plain, params['mer_id'])

    @classmethod
    def mobile_generate_sign(cls, **params):
        '''APP---聚合支付SDK后台生成签名为移动端使用聚合支付SDK签名使用'''
        params = params or {}

        requiredKeys = ['merId', 'orderId', 'orderDate', 'amount']
        for requiredKey in requiredKeys:
            if requiredKey not in params or params[requiredKey] is None or params[requiredKey] == '':
                raise error.ParamsError('签名参数%s为空,请传入%s' % (requiredKey, requiredKey))

        plain = "%s%s%s%s" % (params['merId'], params['orderId'], params['amount'], params['orderDate'])
        sign_str = sign.app_sign(plain, params['merId'])
        util.logger.info("[generate_sign]签名后密文串:%s" % sign_str)
        return sign_str

    @classmethod
    def verify_sign(cls, plain, sign_str):
        '''SDK验签方法传入参数分别为联动返回明文和密文sign字段，明文在前'''
        return sign.verify(plain, sign_str)


# -*- coding: utf-8 -*-

import os
import sys
import warnings

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

try:
    from distutils.command.build_py import build_py_2to3 as build_py
except ImportError:
    from distutils.command.build_py import build_py

path, script = os.path.split(sys.argv[0])
os.chdir(os.path.abspath(path))

install_requires = []

if sys.version_info < (2, 6):
    warnings.warn(
        'Python 2.5 is not officially supported by umfpayservice. ',
        DeprecationWarning)
    install_requires.append('requests >= 0.8.8, < 0.10.1')
    install_requires.append('ssl')
else:
    install_requires.append('requests >= 0.8.8')
    install_requires.append('pycrypto >= 2.6.1')

# Don't import umfpayservice module here, since deps may not be installed
sys.path.insert(0, os.path.join(os.path.dirname(__file__), 'umfpayservice'))
from version import VERSION  # noqa

# Get BeautifulSoup if we don't already have BeautifulSoup
try:
    from bs4 import BeautifulSoup  # noqa
except ImportError:
    install_requires.append('beautifulsoup4')

# Get simplejson if we don't already have json
# if sys.version_info < (3, 0):
#     try:
#         from util import json  # noqa
#     except ImportError:
#         install_requires.append('simplejson')


setup(
    name='umfpayservice',
    packages=["umfpayservice"],
    cmdclass={'build_py': build_py},
    version=VERSION,
    description='umfpayservice python bindings',
    author='Umf',
    author_email='hanpeidong@umpay.com',
    url='https://xy.umfintech.com/spEnterprise/',
    package_data={'umfpayservice': ['../VERSION']},
    install_requires=install_requires,
    use_2to3=True,
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
    ])



# -*- coding: utf-8 -*-
# Exceptions

from umfpayservice import util

class UmfPayError(Exception):
    '''
    umfpayservice错误基类
    '''
    def __init__(self, message=None, http_body=None, http_status=None,
                 resp=None):
        util.logger.error(message)
        super(UmfPayError, self).__init__(message)

        self.http_body = http_body

        self.http_status = http_status
        self.resp = resp

class ParamsError(UmfPayError):
    '''
    参数错误：
    缺少参数、
    参数校验不通过等
    '''
    def __init__(self, message=None, params=None, http_body=None, http_status=None,
                 resp=None):
        super(ParamsError, self).__init__(message, http_body, http_status, resp)

        self.params = params

class SignatureError(UmfPayError):
    '''
    签名错误
    '''
    pass


class APIError(UmfPayError):
    '''
    接口错误：
    访问接口返回异常等
    '''
    pass

class RequestError(UmfPayError):
    '''
    请求错误
    python的请求框架报的错误
    '''
    pass

class ConfigError(UmfPayError):
    '''
    配置错误
    '''
    def __init__(self, message=None, config_key=None, config_value=None):
        super(ConfigError, self).__init__(message)

        self.config_key = config_key
        self.config_value = config_value